@@ -30,7 +30,7 @@ import java.lang.reflect.Modifier;
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Alex Tkachman
- * @version $Revision: 11643 $
+ * @version $Revision: 18226 $
  */
 public abstract class MetaMethod extends ParameterTypes implements Cloneable {
     private String signature;
@@ -61,7 +61,7 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
      */
     public void checkParameters(Class[] arguments) {
         // lets check that the argument types are valid
-        if (!MetaClassHelper.isValidMethod(getParameterTypes(), arguments)) {
+        if (!isValidMethod(arguments)) {
             throw new IllegalArgumentException(
                     "Parameters to method: "
                     + getName()
@@ -71,6 +71,7 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
                     + InvokerHelper.toString(arguments));
         }
     }
+
     public boolean isMethod(MetaMethod method) {
         return getName().equals(method.getName())
             && getModifiers() == method.getModifiers()
@@ -81,7 +82,7 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
     protected static boolean equal(CachedClass[] a, Class[] b) {
         if (a.length == b.length) {
             for (int i = 0, size = a.length; i < size; i++) {
-                if (!a[i].getCachedClass().equals(b[i])) {
+                if (!a[i].getTheClass().equals(b[i])) {
                     return false;
                 }
             }
@@ -203,20 +204,20 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
         return mopName;
     }
 
-    protected final RuntimeException processDoMethodInvokeException (Exception e, Object object, Object [] argumentArray) {
-        if (e instanceof IllegalArgumentException) {
-            //TODO: test if this is ok with new MOP, should be changed!
-            // we don't want the exception being unwrapped if it is a IllegalArgumentException
-            // but in the case it is for example a IllegalThreadStateException, we want the unwrapping
-            // from the runtime
-            //Note: the reason we want unwrapping sometimes and sometimes not is that the method
-            // invokation tries to invoke the method with and then reacts with type transformation
-            // if the invokation failed here. This is ok for IllegalArgumentException, but it is
-            // possible that a Reflector will be used to execute the call and then an Exception from inside
-            // the method is not wrapped in a InvocationTargetException and we will end here.
-            boolean setReason = e.getClass() != IllegalArgumentException.class;
-            return MetaClassHelper.createExceptionText("failed to invoke method: ", this, object, argumentArray, e, setReason);
-        }
+    public final RuntimeException processDoMethodInvokeException (Exception e, Object object, Object [] argumentArray) {
+//        if (e instanceof IllegalArgumentException) {
+//            //TODO: test if this is OK with new MOP, should be changed!
+//            // we don't want the exception being unwrapped if it is a IllegalArgumentException
+//            // but in the case it is for example a IllegalThreadStateException, we want the unwrapping
+//            // from the runtime
+//            //Note: the reason we want unwrapping sometimes and sometimes not is that the method
+//            // invocation tries to invoke the method with and then reacts with type transformation
+//            // if the invocation failed here. This is OK for IllegalArgumentException, but it is
+//            // possible that a Reflector will be used to execute the call and then an Exception from inside
+//            // the method is not wrapped in a InvocationTargetException and we will end here.
+//            boolean setReason = e.getClass() != IllegalArgumentException.class || this instanceof org.codehaus.groovy.reflection.GeneratedMetaMethod;
+//            return MetaClassHelper.createExceptionText("failed to invoke method: ", this, object, argumentArray, e, setReason);
+//        }
 
         if (e instanceof RuntimeException)
           return (RuntimeException) e;
@@ -224,6 +225,8 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
         return MetaClassHelper.createExceptionText("failed to invoke method: ", this, object, argumentArray, e, true);
     }
 
+    // This method is not final but it should be overloaded very carefully and only by generated methods
+    // there is no guarantee that it will be called
     public Object doMethodInvoke(Object object, Object[] argumentArray) {
         argumentArray = coerceArgumentsToClasses(argumentArray);
         try {
