@@ -59,12 +59,14 @@ import org.codehaus.groovy.tools.gse.StringSetMap;
  */
 public class GroovyScriptEngine implements ResourceConnector {
 
-    private static final ClassLoader CL_STUB = new ClassLoader(){};
-    
+    private static final ClassLoader CL_STUB = new ClassLoader() {
+    };
+
     private static WeakReference<ThreadLocal<StringSetMap>> dependencyCache = new WeakReference<ThreadLocal<StringSetMap>>(null);
+
     private static synchronized ThreadLocal<StringSetMap> getDepCache() {
         ThreadLocal<StringSetMap> local = dependencyCache.get();
-        if (local!=null) return local;
+        if (local != null) return local;
         local = new ThreadLocal<StringSetMap>() {
             @Override
             protected StringSetMap initialValue() {
@@ -74,11 +76,12 @@ public class GroovyScriptEngine implements ResourceConnector {
         dependencyCache = new WeakReference<ThreadLocal<StringSetMap>>(local);
         return local;
     }
-    
+
     private static WeakReference<ThreadLocal<CompilationUnit>> localCu = new WeakReference<ThreadLocal<CompilationUnit>>(null);
+
     private static synchronized ThreadLocal<CompilationUnit> getLocalCompilationUnit() {
         ThreadLocal<CompilationUnit> local = localCu.get();
-        if (local!=null) return local;
+        if (local != null) return local;
         local = new ThreadLocal<CompilationUnit>();
         localCu = new WeakReference<ThreadLocal<CompilationUnit>>(local);
         return local;
@@ -92,11 +95,12 @@ public class GroovyScriptEngine implements ResourceConnector {
     private CompilerConfiguration config = new CompilerConfiguration(CompilerConfiguration.DEFAULT);
 
     //TODO: more finals?
+
     private static class ScriptCacheEntry {
         private final Class scriptClass;
         private final long lastModified;
         private final Set<String> dependencies;
-        
+
         public ScriptCacheEntry(Class clazz, long modified, Set<String> depend) {
             this.scriptClass = clazz;
             this.lastModified = modified;
@@ -104,151 +108,156 @@ public class GroovyScriptEngine implements ResourceConnector {
         }
     }
 
-   private class ScriptClassLoader extends GroovyClassLoader {
-       public ScriptClassLoader(GroovyClassLoader loader) {
-           super(loader);
-           setResLoader();
-       }
-       
-       public ScriptClassLoader(ClassLoader loader) {
-           super(loader);
-           setResLoader();
-       }
-       
-       private void setResLoader(){
+    private class ScriptClassLoader extends GroovyClassLoader {
+        public ScriptClassLoader(GroovyClassLoader loader) {
+            super(loader);
+            setResLoader();
+        }
+
+        public ScriptClassLoader(ClassLoader loader) {
+            super(loader);
+            setResLoader();
+        }
+
+        private void setResLoader() {
             final GroovyResourceLoader rl = getResourceLoader();
-            setResourceLoader(new GroovyResourceLoader(){
+            setResourceLoader(new GroovyResourceLoader() {
                 public URL loadGroovySource(String className) throws MalformedURLException {
-                    String filename =   className.replace('.', File.separatorChar) + 
-                                        config.getDefaultScriptExtension();
-                    try {
-                        URLConnection dependentScriptConn = rc.getResourceConnection(filename);
-                        return dependentScriptConn.getURL();
-                    } catch (ResourceException e) {
-                        //TODO: maybe do something here?
-                    }
+                	String filename;
+                	for (String extension : getConfig().getScriptExtensions()) {
+                        filename = className.replace('.', File.separatorChar) + "." + extension;
+		                try {
+		                    URLConnection dependentScriptConn = rc.getResourceConnection(filename);
+		                    return dependentScriptConn.getURL();
+		                } catch (ResourceException e) {
+		                    //TODO: maybe do something here?
+		                }
+                	}
                     return rl.loadGroovySource(className);
                 }
             });
-       }
-       
+        }
+
         @Override
         protected CompilationUnit createCompilationUnit(CompilerConfiguration config, CodeSource source) {
             CompilationUnit cu = super.createCompilationUnit(config, source);
             getLocalCompilationUnit().set(cu);
             final StringSetMap cache = getDepCache().get();
-            
+
             // "." is used to transfer compilation dependencies, which will be
             // recollected later during compilation
             for (String depSourcePath : cache.get(".")) {
                 try {
-                    cu.addSource(new URL("file","",depSourcePath));
-                } catch (MalformedURLException e) {}
+                    cu.addSource(getResourceConnection(depSourcePath).getURL());
+                } catch (ResourceException e) {
+                    /* ignore */
+                }
             }
-            
+
             // remove all old entries including the "." entry
             cache.clear();
             cu.addPhaseOperation(new CompilationUnit.PrimaryClassNodeOperation() {
                 @Override
-                public void call(final SourceUnit source, GeneratorContext context, ClassNode classNode) 
-                    throws CompilationFailedException 
-                {   
-                	// GROOVY-4013: If it is an inner class, tracking its dependencies doesn't really
-                	// serve any purpose and also interferes with the caching done to track dependencies
-                	if(classNode instanceof InnerClassNode) return;
-                    DependencyTracker dt = new DependencyTracker(source,cache);
+                public void call(final SourceUnit source, GeneratorContext context, ClassNode classNode)
+                        throws CompilationFailedException {
+                    // GROOVY-4013: If it is an inner class, tracking its dependencies doesn't really
+                    // serve any purpose and also interferes with the caching done to track dependencies
+                    if (classNode instanceof InnerClassNode) return;
+                    DependencyTracker dt = new DependencyTracker(source, cache);
                     dt.visitClass(classNode);
                 }
             }, Phases.CLASS_GENERATION);
             return cu;
         }
-        
+
         @Override
         public Class parseClass(GroovyCodeSource codeSource, boolean shouldCacheSource) throws CompilationFailedException {
             // local is kept as hard reference to avoid garbage collection
             ThreadLocal<CompilationUnit> localCu = getLocalCompilationUnit();
             ThreadLocal<StringSetMap> localCache = getDepCache();
-            
+
             // we put the old dependencies into local cache so createCompilationUnit 
             // can pick it up. We put that entry under the name "."
             ScriptCacheEntry origEntry = scriptCache.get(codeSource.getName());
             Set<String> origDep = null;
             if (origEntry != null) origDep = origEntry.dependencies;
-            if (origDep != null) localCache.get().put(".",origDep);
-            
+            if (origDep != null) localCache.get().put(".", origDep);
+
             Class answer = super.parseClass(codeSource, false);
-            
+
             StringSetMap cache = localCache.get();
             cache.makeTransitiveHull();
-            long time = System.currentTimeMillis();
-            for (Map.Entry<String,Set<String>> entry: cache.entrySet()) {
+            long now = System.currentTimeMillis();
+            Set<String> entryNames = new HashSet<String>();
+            for (Map.Entry<String, Set<String>> entry : cache.entrySet()) {
                 String className = entry.getKey();
                 Class clazz = getClassCacheEntry(className);
-                if (clazz==null) continue;
-                
+                if (clazz == null) continue;
+
                 String entryName = getPath(clazz);
-                Set<String> value = convertToPaths(entry.getValue()); 
-                ScriptCacheEntry cacheEntry = new ScriptCacheEntry(clazz,time,value);
-                scriptCache.put(entryName,cacheEntry);
+                if (entryNames.contains(entryName)) continue;
+                entryNames.add(entryName);
+                Set<String> value = convertToPaths(entry.getValue());
+                ScriptCacheEntry cacheEntry = new ScriptCacheEntry(clazz, now, value);
+                scriptCache.put(entryName, cacheEntry);
             }
             cache.clear();
             localCu.set(null);
             return answer;
         }
-        
+
         private String getPath(Class clazz) {
             ThreadLocal<CompilationUnit> localCu = getLocalCompilationUnit();
-            
-            ClassNode classNode = localCu.get().getClassNode(clazz.getCanonicalName());
-            String entryName = classNode.getModule().getContext().getName();
-            return entryName;
+            String name = clazz.getName();
+            ClassNode classNode = localCu.get().getClassNode(name);
+            return classNode.getModule().getContext().getName();
         }
-        
+
         private Set<String> convertToPaths(Set<String> orig) {
             Set<String> ret = new HashSet<String>();
             for (String className : orig) {
                 Class clazz = getClassCacheEntry(className);
-                if (clazz==null) continue;
+                if (clazz == null) continue;
                 ret.add(getPath(clazz));
             }
             return ret;
         }
-   }
-   
-   /**
-    * Simple testing harness for the GSE. Enter script roots as arguments and
-    * then input script names to run them.
-    *
-    * @param urls an array of URLs
-    * @throws Exception if something goes wrong
-    */
-   public static void main(String[] urls) throws Exception {
-       GroovyScriptEngine gse = new GroovyScriptEngine(urls);
-       BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
-       String line;
-       while (true) {
-           System.out.print("groovy> ");
-           if ((line = br.readLine()) == null || line.equals("quit"))
-               break;
-           try {
-               System.out.println(gse.run(line, new Binding()));
-           } catch (Exception e) {
-               e.printStackTrace();
-           }
-       }
-   }
+    }
+
+    /**
+     * Simple testing harness for the GSE. Enter script roots as arguments and
+     * then input script names to run them.
+     *
+     * @param urls an array of URLs
+     * @throws Exception if something goes wrong
+     */
+    public static void main(String[] urls) throws Exception {
+        GroovyScriptEngine gse = new GroovyScriptEngine(urls);
+        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
+        String line;
+        while (true) {
+            System.out.print("groovy> ");
+            if ((line = br.readLine()) == null || line.equals("quit"))
+                break;
+            try {
+                System.out.println(gse.run(line, new Binding()));
+            } catch (Exception e) {
+                e.printStackTrace();
+            }
+        }
+    }
 
     /**
      * Initialize a new GroovyClassLoader with a default or
      * constructor-supplied parentClassLoader.
-     * 
+     *
      * @return the parent classloader used to load scripts
      */
     private GroovyClassLoader initGroovyLoader() {
         return (GroovyClassLoader) AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
                 if (parentLoader instanceof GroovyClassLoader) {
-                    return new ScriptClassLoader((GroovyClassLoader)parentLoader);
+                    return new ScriptClassLoader((GroovyClassLoader) parentLoader);
                 } else {
                     return new ScriptClassLoader(parentLoader);
                 }
@@ -300,13 +309,47 @@ public class GroovyScriptEngine implements ResourceConnector {
             }
         }
 
-        if(se == null) se = new ResourceException("No resource for " + resourceName + " was found");
-        
+        if (se == null) se = new ResourceException("No resource for " + resourceName + " was found");
+
         // If we didn't find anything, report on all the exceptions that occurred.
         if (groovyScriptConn == null) throw se;
         return groovyScriptConn;
     }
 
+    /**
+     * This method closes a {@link URLConnection} by getting its {@link InputStream} and calling the
+     * {@link InputStream#close()} method on it. The {@link URLConnection} doesn't have a close() method
+     * and relies on garbage collection to close the underlying connection to the file.
+     * Relying on garbage collection could lead to the application exhausting the number of files the
+     * user is allowed to have open at any one point in time and cause the application to crash
+     * ({@link FileNotFoundException} (Too many open files)).
+     * Hence the need for this method to explicitly close the underlying connection to the file.
+     *
+     * @param urlConnection the {@link URLConnection} to be "closed" to close the underlying file descriptors.
+     */
+    private void forceClose(URLConnection urlConnection) {
+        if (urlConnection != null) {
+            // We need to get the input stream and close it to force the open
+            // file descriptor to be released. Otherwise, we will reach the limit
+            // for number of files open at one time.
+
+            InputStream in = null;
+            try {
+                in = urlConnection.getInputStream();
+            } catch (Exception e) {
+                // Do nothing: We were not going to use it anyway.
+            } finally {
+                if (in != null) {
+                    try {
+                        in.close();
+                    } catch (IOException e) {
+                        // Do nothing: Just want to make sure it is closed.
+                    }
+                }
+            }
+        }
+    }
+
     /**
      * The groovy script engine will run groovy scripts and reload them and
      * their dependencies when they are modified. This is useful for embedding
@@ -316,33 +359,33 @@ public class GroovyScriptEngine implements ResourceConnector {
      *              be laid out using their package structure like Java classes
      */
     private GroovyScriptEngine(URL[] roots, ClassLoader parent, ResourceConnector rc) {
-        if (roots==null) roots = new URL[0]; 
+        if (roots == null) roots = new URL[0];
         this.roots = roots;
-        if (rc==null) rc = this;
+        if (rc == null) rc = this;
         this.rc = rc;
-        if (parent==CL_STUB) parent = this.getClass().getClassLoader();
+        if (parent == CL_STUB) parent = this.getClass().getClassLoader();
         this.parentLoader = parent;
         this.groovyLoader = initGroovyLoader();
-        for (URL root: roots) this.groovyLoader.addURL(root);
+        for (URL root : roots) this.groovyLoader.addURL(root);
     }
-    
+
     public GroovyScriptEngine(URL[] roots) {
-        this(roots,CL_STUB,null);
+        this(roots, CL_STUB, null);
     }
 
     public GroovyScriptEngine(URL[] roots, ClassLoader parentClassLoader) {
-        this(roots,parentClassLoader,null);
+        this(roots, parentClassLoader, null);
     }
 
     public GroovyScriptEngine(String[] urls) throws IOException {
-        this(createRoots(urls),CL_STUB, null);
+        this(createRoots(urls), CL_STUB, null);
     }
 
     private static URL[] createRoots(String[] urls) throws MalformedURLException {
-        if (urls==null) return null;
+        if (urls == null) return null;
         URL[] roots = new URL[urls.length];
         for (int i = 0; i < roots.length; i++) {
-            if(urls[i].indexOf("://") != -1) {
+            if (urls[i].indexOf("://") != -1) {
                 roots[i] = new URL(urls[i]);
             } else {
                 roots[i] = new File(urls[i]).toURI().toURL();
@@ -352,7 +395,7 @@ public class GroovyScriptEngine implements ResourceConnector {
     }
 
     public GroovyScriptEngine(String[] urls, ClassLoader parentClassLoader) throws IOException {
-        this(createRoots(urls),parentClassLoader, null);
+        this(createRoots(urls), parentClassLoader, null);
     }
 
     public GroovyScriptEngine(String url) throws IOException {
@@ -360,15 +403,15 @@ public class GroovyScriptEngine implements ResourceConnector {
     }
 
     public GroovyScriptEngine(String url, ClassLoader parentClassLoader) throws IOException {
-        this(new String[]{url},parentClassLoader);
+        this(new String[]{url}, parentClassLoader);
     }
 
     public GroovyScriptEngine(ResourceConnector rc) {
-        this(null,CL_STUB, rc);
+        this(null, CL_STUB, rc);
     }
 
     public GroovyScriptEngine(ResourceConnector rc, ClassLoader parentClassLoader) {
-        this(null,parentClassLoader, rc);
+        this(null, parentClassLoader, rc);
     }
 
     /**
@@ -384,58 +427,62 @@ public class GroovyScriptEngine implements ResourceConnector {
 
     /**
      * @param parentClassLoader ClassLoader to be used as the parent ClassLoader
-     *        for scripts executed by the engine
+     *                          for scripts executed by the engine
      * @deprecated
      */
     public void setParentClassLoader(ClassLoader parentClassLoader) {
         throw new DeprecationException(
                 "The method GroovyScriptEngine#setParentClassLoader(ClassLoader) " +
-                "is no longer supported. Specify a parentLoader in the constructor instead."
+                        "is no longer supported. Specify a parentLoader in the constructor instead."
         );
     }
 
     /**
-     * Get the class of the scriptName in question, so that you can instantiate 
+     * Get the class of the scriptName in question, so that you can instantiate
      * Groovy objects with caching and reloading.
      *
      * @param scriptName resource name pointing to the script
      * @return the loaded scriptName as a compiled class
      * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException if there is a problem parsing the script
+     * @throws ScriptException   if there is a problem parsing the script
      */
     public Class loadScriptByName(String scriptName) throws ResourceException, ScriptException {
         URLConnection conn = rc.getResourceConnection(scriptName);
         String path = conn.getURL().getPath();
         ScriptCacheEntry entry = scriptCache.get(path);
         Class clazz = null;
-        if (entry!=null) clazz=entry.scriptClass;
-        if (isSourceNewer(entry)) {
-            try {
-                String encoding = conn.getContentEncoding() != null ? conn.getContentEncoding() : "UTF-8";
-                clazz = groovyLoader.parseClass(DefaultGroovyMethods.getText(conn.getInputStream(), encoding), conn.getURL().getPath());
-            } catch (IOException e) {
-                throw new ResourceException(e);
+        if (entry != null) clazz = entry.scriptClass;
+        try {
+            if (isSourceNewer(entry)) {
+                try {
+                    String encoding = conn.getContentEncoding() != null ? conn.getContentEncoding() : "UTF-8";
+                    clazz = groovyLoader.parseClass(DefaultGroovyMethods.getText(conn.getInputStream(), encoding), path);
+                } catch (IOException e) {
+                    throw new ResourceException(e);
+                }
             }
+        } finally {
+            forceClose(conn);
         }
         return clazz;
     }
 
     /**
-     * Get the class of the scriptName in question, so that you can instantiate 
+     * Get the class of the scriptName in question, so that you can instantiate
      * Groovy objects with caching and reloading.
      *
-     * @param scriptName resource name pointing to the script
+     * @param scriptName        resource name pointing to the script
      * @param parentClassLoader the class loader to use when loading the script
      * @return the loaded scriptName as a compiled class
      * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException if there is a problem parsing the script
+     * @throws ScriptException   if there is a problem parsing the script
      * @deprecated
      */
     public Class loadScriptByName(String scriptName, ClassLoader parentClassLoader)
             throws ResourceException, ScriptException {
         throw new DeprecationException(
-                "The method GroovyScriptEngine#loadScriptByName(String,ClassLoader) "+
-                "is no longer supported. Use GroovyScriptEngine#loadScriptByName(String) instead."
+                "The method GroovyScriptEngine#loadScriptByName(String,ClassLoader) " +
+                        "is no longer supported. Use GroovyScriptEngine#loadScriptByName(String) instead."
         );
     }
 
@@ -446,7 +493,7 @@ public class GroovyScriptEngine implements ResourceConnector {
      * @param argument   a single argument passed as a variable named <code>arg</code> in the binding
      * @return a <code>toString()</code> representation of the result of the execution of the script
      * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException if there is a problem parsing the script
+     * @throws ScriptException   if there is a problem parsing the script
      */
     public String run(String scriptName, String argument) throws ResourceException, ScriptException {
         Binding binding = new Binding();
@@ -462,7 +509,7 @@ public class GroovyScriptEngine implements ResourceConnector {
      * @param binding    the binding to pass to the script
      * @return an object
      * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException if there is a problem parsing the script
+     * @throws ScriptException   if there is a problem parsing the script
      */
     public Object run(String scriptName, Binding binding) throws ResourceException, ScriptException {
         return createScript(scriptName, binding).run();
@@ -475,35 +522,34 @@ public class GroovyScriptEngine implements ResourceConnector {
      * @param binding    the binding to pass to the script
      * @return the script object
      * @throws ResourceException if there is a problem accessing the script
-     * @throws ScriptException if there is a problem parsing the script
+     * @throws ScriptException   if there is a problem parsing the script
      */
     public Script createScript(String scriptName, Binding binding) throws ResourceException, ScriptException {
         return InvokerHelper.createScript(loadScriptByName(scriptName), binding);
     }
 
-    protected boolean isSourceNewer(ScriptCacheEntry entry) throws ResourceException  {
-        if (entry==null) return true;
-        long time = System.currentTimeMillis();
-        
-        for (String scriptName:entry.dependencies) {
+    protected boolean isSourceNewer(ScriptCacheEntry entry) throws ResourceException {
+        if (entry == null) return true;
+        long now = System.currentTimeMillis();
+
+        for (String scriptName : entry.dependencies) {
             ScriptCacheEntry depEntry = scriptCache.get(scriptName);
-            long entryChangeTime = depEntry.lastModified + config.getMinimumRecompilationInterval();
-            if (entryChangeTime>time) continue;
+            long nextPossibleRecompilationTime = depEntry.lastModified + config.getMinimumRecompilationInterval();
+            if (nextPossibleRecompilationTime > now) continue;
 
             URLConnection conn = rc.getResourceConnection(scriptName);
-            URL source = conn.getURL();
-            String path = source.getPath().replace('/', File.separatorChar).replace('|', ':');
-            File file = new File(path);
-            long lastMod = file.lastModified();
+            // getLastModified() truncates up to 999 ms from the true modification time, let's fix that
+            long lastMod = ((conn.getLastModified() / 1000) + 1) * 1000 - 1;
+            // getResourceConnection() opening the inputstream, let's ensure all streams are closed
+            forceClose(conn);
 
-            if (entryChangeTime > lastMod) {
-                ScriptCacheEntry newEntry = new ScriptCacheEntry(depEntry.scriptClass,time,depEntry.dependencies);
-                scriptCache.put(scriptName,newEntry);
-                continue;
+            if (depEntry.lastModified < lastMod) {
+                ScriptCacheEntry newEntry = new ScriptCacheEntry(depEntry.scriptClass, lastMod, depEntry.dependencies);
+                scriptCache.put(scriptName, newEntry);
+                return true;
             }
-            return true;
         }
-        
+
         return false;
     }
 
@@ -516,7 +562,7 @@ public class GroovyScriptEngine implements ResourceConnector {
     public GroovyClassLoader getGroovyClassLoader() {
         return groovyLoader;
     }
-    
+
     /**
      * @return a non null compiler configuration
      */
@@ -526,11 +572,12 @@ public class GroovyScriptEngine implements ResourceConnector {
 
     /**
      * sets a compiler configuration
+     *
      * @param config - the compiler configuration
      * @throws NullPointerException if config is null
      */
     public void setConfig(CompilerConfiguration config) {
-        if (config==null) throw new NullPointerException("configuration cannot be null");
+        if (config == null) throw new NullPointerException("configuration cannot be null");
         this.config = config;
     }
 }
