@@ -17,6 +17,7 @@
 package org.codehaus.groovy.ant;
 
 import groovy.lang.GroovyClassLoader;
+import groovy.lang.GroovyResourceLoader;
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.PosixParser;
@@ -34,6 +35,7 @@ import org.apache.tools.ant.util.GlobPatternMapper;
 import org.apache.tools.ant.util.SourceFileScanner;
 import org.codehaus.groovy.control.CompilationUnit;
 import org.codehaus.groovy.control.CompilerConfiguration;
+import org.codehaus.groovy.control.SourceExtensionHandler;
 import org.codehaus.groovy.tools.ErrorReporter;
 import org.codehaus.groovy.tools.FileSystemCompiler;
 import org.codehaus.groovy.tools.RootLoader;
@@ -45,13 +47,16 @@ import java.io.FileWriter;
 import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
+import java.net.MalformedURLException;
 import java.net.URL;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Iterator;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.StringTokenizer;
 
 /**
@@ -82,7 +87,7 @@ import java.util.StringTokenizer;
  * @author Hein Meling
  * @author <a href="mailto:russel.winder@concertant.com">Russel Winder</a>
  * @author Danno Ferrin
- * @version $Revision: 20356 $
+ * @version $Revision: 21452 $
  */
 public class Groovyc extends MatchingTask {
     private final LoggingHelper log = new LoggingHelper(this);
@@ -119,6 +124,8 @@ public class Groovyc extends MatchingTask {
     private List<File> temporaryFiles = new ArrayList(2);
     private File stubDir;
     private boolean keepStubs;
+    
+    private Set<String> scriptExtensions = new LinkedHashSet<String>();
 
 
     /**
@@ -580,7 +587,8 @@ public class Groovyc extends MatchingTask {
     public void execute() throws BuildException {
         checkParameters();
         resetFileLists();
-
+        loadRegisteredScriptExtensions();
+        
         if (javac != null) jointCompilation = true;
 
         // scan source directories and dest directory to build up
@@ -609,6 +617,7 @@ public class Groovyc extends MatchingTask {
      */
     protected void resetFileLists() {
         compileList = new File[0];
+        scriptExtensions = new LinkedHashSet<String>();
     }
 
     /**
@@ -621,11 +630,14 @@ public class Groovyc extends MatchingTask {
      */
     protected void scanDir(File srcDir, File destDir, String[] files) {
         GlobPatternMapper m = new GlobPatternMapper();
-        m.setFrom(scriptExtension);
-        m.setTo("*.class");
         SourceFileScanner sfs = new SourceFileScanner(this);
-        File[] newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
-        addToCompileList(newFiles);
+        File[] newFiles;
+        for (String extension : getScriptExtensions()) {
+        	m.setFrom("*." + extension);
+        	m.setTo("*.class");
+        	newFiles = sfs.restrictAsFiles(files, srcDir, destDir, m);
+        	addToCompileList(newFiles);
+        }
 
         if (jointCompilation) {
             m.setFrom("*.java");
@@ -770,7 +782,7 @@ public class Groovyc extends MatchingTask {
                     if ((fileEncodingProp != null) && !fileEncodingProp.equals("")) {
                         commandLineList.add("-Dfile.encoding=" + fileEncodingProp);
                     }
-                    if ((targetBytecode != null)) {
+                    if (targetBytecode != null) {
                         commandLineList.add("-Dgroovy.target.bytecode=" + targetBytecode);
                     }
 
@@ -780,6 +792,11 @@ public class Groovyc extends MatchingTask {
                     if ((memoryMaximumSize != null) && !memoryMaximumSize.equals("")) {
                         commandLineList.add("-Xmx" + memoryMaximumSize);
                     }
+                    if(!"*.groovy".equals(getScriptExtension())) {
+                    	String tmpExtension = getScriptExtension();
+                    	if(tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
+                    	commandLineList.add("-Dgroovy.default.scriptExtension=" + tmpExtension);
+                    }
                     commandLineList.add("org.codehaus.groovy.tools.FileSystemCompiler");
                 }
                 commandLineList.add("--classpath");
@@ -869,6 +886,10 @@ public class Groovyc extends MatchingTask {
                         cli = cliParser.parse(options, commandLine);
 
                         configuration = FileSystemCompiler.generateCompilerConfigurationFromOptions(cli);
+                        configuration.setScriptExtensions(getScriptExtensions());
+                    	String tmpExtension = getScriptExtension();
+                    	if(tmpExtension.startsWith("*.")) tmpExtension = tmpExtension.substring(1);
+                        configuration.setDefaultScriptExtension(tmpExtension);
 
                         // Load the file name list
                         String[] filenames = FileSystemCompiler.generateFileNamesFromOptions(cli);
@@ -877,7 +898,7 @@ public class Groovyc extends MatchingTask {
                         fileNameErrors = fileNameErrors && !FileSystemCompiler.validateFiles(filenames);
 
                         if(targetBytecode != null) {
-                        	configuration.setTargetBytecode(targetBytecode);
+                            configuration.setTargetBytecode(targetBytecode);
                         }
                         
                         if (!fileNameErrors) {
@@ -968,7 +989,16 @@ public class Groovyc extends MatchingTask {
                     antLoader.addPathElement(cpEntry);
             }
         }
-        return new GroovyClassLoader(parent, configuration);
+
+        GroovyClassLoader loader = new GroovyClassLoader(parent, configuration);
+        // in command line we don't need to do script lookups
+        loader.setResourceLoader(new GroovyResourceLoader() {
+            public URL loadGroovySource(String filename) throws MalformedURLException {
+                return null;
+            }
+        });
+
+        return loader;
     }
 
     /**
@@ -1012,4 +1042,23 @@ public class Groovyc extends MatchingTask {
     public boolean getKeepStubs() {
         return keepStubs;
     }
+    
+    private Set<String> getScriptExtensions() {
+    	return scriptExtensions;
+    }
+
+    private void loadRegisteredScriptExtensions() {
+    	if (scriptExtensions.isEmpty()) {
+    		
+    		scriptExtensions.add(getScriptExtension().substring(2)); // first extension will be the one set explicitly on <groovyc>
+    		
+    		Path classpath = getClasspath() != null ? getClasspath() : new Path(getProject());
+    		final String[] pe = classpath.list();
+    		final GroovyClassLoader loader = new GroovyClassLoader(getClass().getClassLoader());
+    		for (String file : pe) {
+    			loader.addClasspath(file);
+    		}
+    		scriptExtensions.addAll(SourceExtensionHandler.getRegisteredExtensions(loader));
+    	}
+    }
 }
