@@ -1,9 +1,9 @@
 /*******************************************************************************
  * Copyright (c) 2004 IBM Corporation and others.
- * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
- * which is available at:
+ * All rights reserved.   The initial API is made available under the terms of
+ * the Common Public License v1.0 which is available at:
  * http://www.eclipse.org/legal/cpl-v10.html
+ * Subsequent modifications are made available under the Apache 2.0 license.
  *
  * Contributors:
  * IBM - Initial API and implementation
@@ -11,8 +11,6 @@
  ******************************************************************************/
 package org.codehaus.groovy.classgen;
 
-import java.lang.reflect.Modifier;
-import java.util.Iterator;
 import java.util.List;
 
 import org.codehaus.groovy.ast.ClassCodeVisitorSupport;
@@ -31,6 +29,7 @@ import org.codehaus.groovy.ast.expr.Expression;
 import org.codehaus.groovy.ast.expr.GStringExpression;
 import org.codehaus.groovy.ast.expr.MapEntryExpression;
 import org.codehaus.groovy.ast.expr.MethodCallExpression;
+import org.codehaus.groovy.ast.expr.PropertyExpression;
 import org.codehaus.groovy.ast.expr.TupleExpression;
 import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.stmt.CatchStatement;
@@ -39,6 +38,8 @@ import org.objectweb.asm.Opcodes;
 import org.codehaus.groovy.runtime.MetaClassHelper;
 import org.codehaus.groovy.syntax.Types;
 
+import static java.lang.reflect.Modifier.*;
+
 /**
  * ClassCompletionVerifier
  */
@@ -65,6 +66,7 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
             checkClassForIncorrectModifiers(node);
             checkClassForOverwritingFinal(node);
             checkMethodsForIncorrectModifiers(node);
+            checkMethodsForWeakerAccess(node);
             checkMethodsForOverridingFinal(node);
             checkNoAbstractMethodsNonabstractClass(node);
         }
@@ -73,11 +75,10 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     private void checkNoAbstractMethodsNonabstractClass(ClassNode node) {
-        if (Modifier.isAbstract(node.getModifiers())) return;
-        List abstractMethods = node.getAbstractMethods();
+        if (isAbstract(node.getModifiers())) return;
+        List<MethodNode> abstractMethods = node.getAbstractMethods();
         if (abstractMethods == null) return;
-        for (Iterator iter = abstractMethods.iterator(); iter.hasNext();) {
-            MethodNode method = (MethodNode) iter.next();
+        for (MethodNode method : abstractMethods) {
             addError("Can't have an abstract method in a non-abstract class." +
                     " The " + getDescription(node) + " must be declared abstract or" +
                     " the " + getDescription(method) + " must be implemented.", node);
@@ -90,19 +91,19 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     private void checkClassForAbstractAndFinal(ClassNode node) {
-        if (!Modifier.isAbstract(node.getModifiers())) return;
-        if (!Modifier.isFinal(node.getModifiers())) return;
+        if (!isAbstract(node.getModifiers())) return;
+        if (!isFinal(node.getModifiers())) return;
         if (node.isInterface()) {
-            addError("The " + getDescription(node) +" must not be final. It is by definition abstract.", node);
+            addError("The " + getDescription(node) + " must not be final. It is by definition abstract.", node);
         } else {
             addError("The " + getDescription(node) + " must not be both final and abstract.", node);
         }
     }
 
     private void checkClassForOtherModifiers(ClassNode node) {
-        checkClassForModifier(node, Modifier.isTransient(node.getModifiers()), "transient");
-        checkClassForModifier(node, Modifier.isVolatile(node.getModifiers()), "volatile");
-        checkClassForModifier(node, Modifier.isNative(node.getModifiers()), "native");
+        checkClassForModifier(node, isTransient(node.getModifiers()), "transient");
+        checkClassForModifier(node, isVolatile(node.getModifiers()), "volatile");
+        checkClassForModifier(node, isNative(node.getModifiers()), "native");
         // don't check synchronized here as it overlaps with ACC_SUPER
     }
 
@@ -129,8 +130,8 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     private void checkAbstractDeclaration(MethodNode methodNode) {
-        if (!Modifier.isAbstract(methodNode.getModifiers())) return;
-        if (Modifier.isAbstract(currentClass.getModifiers())) return;
+        if (!isAbstract(methodNode.getModifiers())) return;
+        if (isAbstract(currentClass.getModifiers())) return;
         addError("Can't have an abstract method in a non-abstract class." +
                 " The " + getDescription(currentClass) + " must be declared abstract or the method '" +
                 methodNode.getTypeDescriptor() + "' must not be abstract.", methodNode);
@@ -139,7 +140,7 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     private void checkClassForOverwritingFinal(ClassNode cn) {
         ClassNode superCN = cn.getSuperClass();
         if (superCN == null) return;
-        if (!Modifier.isFinal(superCN.getModifiers())) return;
+        if (!isFinal(superCN.getModifiers())) return;
         StringBuffer msg = new StringBuffer();
         msg.append("You are not allowed to overwrite the final ");
         msg.append(getDescription(superCN));
@@ -152,9 +153,8 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
         if (cn.isInterface() && !node.isInterface()) {
             addError("You are not allowed to extend the " + getDescription(cn) + ", use implements instead.", node);
         }
-        ClassNode[] interfaces = node.getInterfaces();
-        for (int i = 0; i < interfaces.length; i++) {
-            cn = interfaces[i];
+        for (ClassNode anInterface : node.getInterfaces()) {
+            cn = anInterface;
             if (!cn.isInterface()) {
                 addError("You are not allowed to implement the " + getDescription(cn) + ", use extends instead.", node);
             }
@@ -163,35 +163,35 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
 
     private void checkMethodsForIncorrectModifiers(ClassNode cn) {
         if (!cn.isInterface()) return;
-        List methods = cn.getMethods();
-        for (Iterator cnIter = methods.iterator(); cnIter.hasNext();) {
-            MethodNode method = (MethodNode) cnIter.next();
-            if (Modifier.isFinal(method.getModifiers())) {
+        for (MethodNode method : cn.getMethods()) {
+            if (isFinal(method.getModifiers())) {
                 addError("The " + getDescription(method) + " from " + getDescription(cn) +
                         " must not be final. It is by definition abstract.", method);
             }
-            if (Modifier.isStatic(method.getModifiers()) && !isConstructor(method)) {
+            if (isStatic(method.getModifiers()) && !isConstructor(method)) {
                 addError("The " + getDescription(method) + " from " + getDescription(cn) +
                         " must not be static. Only fields may be static in an interface.", method);
             }
         }
     }
 
+    private void checkMethodsForWeakerAccess(ClassNode cn) {
+        for (MethodNode method : cn.getMethods()) {
+            checkMethodForWeakerAccessPrivileges(method, cn);
+        }
+    }
+
     private boolean isConstructor(MethodNode method) {
         return method.getName().equals("<clinit>");
     }
 
     private void checkMethodsForOverridingFinal(ClassNode cn) {
-        List methods = cn.getMethods();
-        for (Iterator cnIter = methods.iterator(); cnIter.hasNext();) {
-            MethodNode method = (MethodNode) cnIter.next();
+        for (MethodNode method : cn.getMethods()) {
             Parameter[] params = method.getParameters();
-            List superMethods = cn.getSuperClass().getMethods(method.getName());
-            for (Iterator iter = superMethods.iterator(); iter.hasNext();) {
-                MethodNode superMethod = (MethodNode) iter.next();
+            for (MethodNode superMethod : cn.getSuperClass().getMethods(method.getName())) {
                 Parameter[] superParams = superMethod.getParameters();
                 if (!hasEqualParameterTypes(params, superParams)) continue;
-                if (!Modifier.isFinal(superMethod.getModifiers())) break;
+                if (!isFinal(superMethod.getModifiers())) break;
                 addInvalidUseOfFinalError(method, params, superMethod.getDeclaringClass());
                 return;
             }
@@ -203,19 +203,43 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
         msg.append("You are not allowed to override the final method ").append(method.getName());
         msg.append("(");
         boolean needsComma = false;
-        for (int i = 0; i < parameters.length; i++) {
+        for (Parameter parameter : parameters) {
             if (needsComma) {
                 msg.append(",");
             } else {
                 needsComma = true;
             }
-            msg.append(parameters[i].getType());
+            msg.append(parameter.getType());
         }
         msg.append(") from ").append(getDescription(superCN));
         msg.append(".");
         addError(msg.toString(), method);
     }
 
+    private void addWeakerAccessError(ClassNode cn, MethodNode method, Parameter[] parameters, MethodNode superMethod) {
+        StringBuffer msg = new StringBuffer();
+        msg.append(method.getName());
+        msg.append("(");
+        boolean needsComma = false;
+        for (Parameter parameter : parameters) {
+            if (needsComma) {
+                msg.append(",");
+            } else {
+                needsComma = true;
+            }
+            msg.append(parameter.getType());
+        }
+        msg.append(") in ");
+        msg.append(cn.getName());
+        msg.append(" cannot override ");
+        msg.append(superMethod.getName());
+        msg.append(" in ");
+        msg.append(superMethod.getDeclaringClass().getName());
+        msg.append("; attempting to assign weaker access privileges; was ");
+        msg.append(isPublic(superMethod.getModifiers()) ? "public" : "protected");
+        addError(msg.toString(), method);
+    }
+
     private boolean hasEqualParameterTypes(Parameter[] first, Parameter[] second) {
         if (first.length != second.length) return false;
         for (int i = 0; i < first.length; i++) {
@@ -232,8 +256,8 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     public void visitMethod(MethodNode node) {
-    	inConstructor = false;
-    	inStaticConstructor = node.isStaticConstructor();
+        inConstructor = false;
+        inStaticConstructor = node.isStaticConstructor();
         checkAbstractDeclaration(node);
         checkRepetitiveMethod(node);
         checkOverloadingPrivateAndPublic(node);
@@ -245,44 +269,53 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
         // don't check volatile here as it overlaps with ACC_BRIDGE
         // additional modifiers not allowed for interfaces
         if ((this.currentClass.getModifiers() & Opcodes.ACC_INTERFACE) != 0) {
-            checkMethodForModifier(node, Modifier.isStrict(node.getModifiers()), "strictfp");
-            checkMethodForModifier(node, Modifier.isSynchronized(node.getModifiers()), "synchronized");
-            checkMethodForModifier(node, Modifier.isNative(node.getModifiers()), "native");
+            checkMethodForModifier(node, isStrict(node.getModifiers()), "strictfp");
+            checkMethodForModifier(node, isSynchronized(node.getModifiers()), "synchronized");
+            checkMethodForModifier(node, isNative(node.getModifiers()), "native");
+        }
+    }
+
+    private void checkMethodForWeakerAccessPrivileges(MethodNode mn, ClassNode cn) {
+        Parameter[] params = mn.getParameters();
+        for (MethodNode superMethod : cn.getSuperClass().getMethods(mn.getName())) {
+            Parameter[] superParams = superMethod.getParameters();
+            if (!hasEqualParameterTypes(params, superParams)) continue;
+            if ((isPrivate(mn.getModifiers()) && !isPrivate(superMethod.getModifiers())) ||
+                    (isProtected(mn.getModifiers()) && isPublic(superMethod.getModifiers()))) {
+                addWeakerAccessError(cn, mn, params, superMethod);
+                return;
+            }
         }
     }
 
     private void checkOverloadingPrivateAndPublic(MethodNode node) {
         if (isConstructor(node)) return;
-        List methods = currentClass.getMethods(node.getName());
-        boolean hasPrivate=false;
-        boolean hasPublic=false;
-        for (Iterator iter = methods.iterator(); iter.hasNext();) {
-            MethodNode element = (MethodNode) iter.next();
-            if (element == node) continue;
-            if (!element.getDeclaringClass().equals(node.getDeclaringClass())) continue;
-            int modifiers = element.getModifiers();
-            if (Modifier.isPublic(modifiers) || Modifier.isProtected(modifiers)){
-                hasPublic=true;
+        boolean hasPrivate = false;
+        boolean hasPublic = false;
+        for (MethodNode method : currentClass.getMethods(node.getName())) {
+            if (method == node) continue;
+            if (!method.getDeclaringClass().equals(node.getDeclaringClass())) continue;
+            int modifiers = method.getModifiers();
+            if (isPublic(modifiers) || isProtected(modifiers)) {
+                hasPublic = true;
             } else {
-                hasPrivate=true;
+                hasPrivate = true;
             }
         }
         if (hasPrivate && hasPublic) {
-            addError("Mixing private and public/protected methods of the same name causes multimethods to be disabled and is forbidden to avoid surprising behaviour. Renaming the private methods will solve the problem.",node);
+            addError("Mixing private and public/protected methods of the same name causes multimethods to be disabled and is forbidden to avoid surprising behaviour. Renaming the private methods will solve the problem.", node);
         }
     }
-    
+
     private void checkRepetitiveMethod(MethodNode node) {
         if (isConstructor(node)) return;
-        List methods = currentClass.getMethods(node.getName());
-        for (Iterator iter = methods.iterator(); iter.hasNext();) {
-            MethodNode element = (MethodNode) iter.next();
-            if (element == node) continue;
-            if (!element.getDeclaringClass().equals(node.getDeclaringClass())) continue;
+        for (MethodNode method : currentClass.getMethods(node.getName())) {
+            if (method == node) continue;
+            if (!method.getDeclaringClass().equals(node.getDeclaringClass())) continue;
             Parameter[] p1 = node.getParameters();
-            Parameter[] p2 = element.getParameters();
+            Parameter[] p2 = method.getParameters();
             if (p1.length != p2.length) continue;
-            addErrorIfParamsAndReturnTypeEqual(p2, p1, node, element);
+            addErrorIfParamsAndReturnTypeEqual(p2, p1, node, method);
         }
     }
 
@@ -308,27 +341,27 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     public void visitProperty(PropertyNode node) {
-    	checkDuplicateProperties(node);
-    	super.visitProperty(node);
+        checkDuplicateProperties(node);
+        super.visitProperty(node);
     }
-    
+
     private void checkDuplicateProperties(PropertyNode node) {
-    	ClassNode cn = node.getDeclaringClass();
-    	String name = node.getName();
-    	String getterName = "get" + MetaClassHelper.capitalize(name);
-    	if(Character.isUpperCase(name.charAt(0))) {
-    		for (PropertyNode propNode : cn.getProperties()) {
-    			String otherName = propNode.getField().getName();
-    			String otherGetterName = "get" + MetaClassHelper.capitalize(otherName);
-    			if(node != propNode && getterName.equals(otherGetterName)) {
-    				String msg = "The field " + name + " and " + otherName + " on the class " +
-    				cn.getName() + " will result in duplicate JavaBean properties, which is not allowed";
-    				addError(msg, node);
-    			}
-    		}
-    	}
+        ClassNode cn = node.getDeclaringClass();
+        String name = node.getName();
+        String getterName = "get" + MetaClassHelper.capitalize(name);
+        if (Character.isUpperCase(name.charAt(0))) {
+            for (PropertyNode propNode : cn.getProperties()) {
+                String otherName = propNode.getField().getName();
+                String otherGetterName = "get" + MetaClassHelper.capitalize(otherName);
+                if (node != propNode && getterName.equals(otherGetterName)) {
+                    String msg = "The field " + name + " and " + otherName + " on the class " +
+                            cn.getName() + " will result in duplicate JavaBean properties, which is not allowed";
+                    addError(msg, node);
+                }
+            }
+        }
     }
-    
+
     private void checkInterfaceFieldModifiers(FieldNode node) {
         if (!currentClass.isInterface()) return;
         if ((node.getModifiers() & (Opcodes.ACC_PUBLIC | Opcodes.ACC_STATIC | Opcodes.ACC_FINAL)) == 0) {
@@ -346,31 +379,44 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
         }
         super.visitBinaryExpression(expression);
 
-        switch (expression.getOperation().getType()){
-	        case Types.EQUAL: // = assignment
-	        case Types.BITWISE_AND_EQUAL:
-	        case Types.BITWISE_OR_EQUAL:
-	        case Types.BITWISE_XOR_EQUAL:
-	        case Types.PLUS_EQUAL:
-	        case Types.MINUS_EQUAL:
-	        case Types.MULTIPLY_EQUAL:
-	        case Types.DIVIDE_EQUAL:
-	        case Types.INTDIV_EQUAL:
-	        case Types.MOD_EQUAL:
-	        case Types.POWER_EQUAL:
-	        case Types.LEFT_SHIFT_EQUAL:
-	        case Types.RIGHT_SHIFT_EQUAL:
-	        case Types.RIGHT_SHIFT_UNSIGNED_EQUAL:
-	            checkFinalFieldAccess(expression.getLeftExpression());
-	            break;
-	        default: break;
+        switch (expression.getOperation().getType()) {
+            case Types.EQUAL: // = assignment
+            case Types.BITWISE_AND_EQUAL:
+            case Types.BITWISE_OR_EQUAL:
+            case Types.BITWISE_XOR_EQUAL:
+            case Types.PLUS_EQUAL:
+            case Types.MINUS_EQUAL:
+            case Types.MULTIPLY_EQUAL:
+            case Types.DIVIDE_EQUAL:
+            case Types.INTDIV_EQUAL:
+            case Types.MOD_EQUAL:
+            case Types.POWER_EQUAL:
+            case Types.LEFT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_UNSIGNED_EQUAL:
+                checkFinalFieldAccess(expression.getLeftExpression());
+                break;
+            default:
+                break;
         }
     }
 
     private void checkFinalFieldAccess(Expression expression) {
-        if (!(expression instanceof VariableExpression)) return;
-        VariableExpression ve = (VariableExpression) expression;
-        Variable v = ve.getAccessedVariable();
+        if (!(expression instanceof VariableExpression) && !(expression instanceof PropertyExpression)) return;
+        Variable v = null;
+        if (expression instanceof VariableExpression) {
+            VariableExpression ve = (VariableExpression) expression;
+            v = ve.getAccessedVariable();
+        } else {
+        	PropertyExpression propExp = ((PropertyExpression) expression);
+        	Expression objectExpression = propExp.getObjectExpression();
+        	if(objectExpression instanceof VariableExpression) {
+        		VariableExpression varExp = (VariableExpression) objectExpression;
+        		if(varExp.isThisExpression()) {
+        			v = currentClass.getDeclaredField(propExp.getPropertyAsString());
+        		}
+        	}
+        }
         if (v instanceof FieldNode) {
             FieldNode fn = (FieldNode) v;
             int modifiers = fn.getModifiers();
@@ -389,56 +435,56 @@ public class ClassCompletionVerifier extends ClassCodeVisitorSupport {
     }
 
     public void visitConstructor(ConstructorNode node) {
-    	inConstructor = true;
-    	inStaticConstructor = node.isStaticConstructor();
+        inConstructor = true;
+        inStaticConstructor = node.isStaticConstructor();
         super.visitConstructor(node);
     }
-    
+
     public void visitCatchStatement(CatchStatement cs) {
         if (!(cs.getExceptionType().isDerivedFrom(ClassHelper.make(Throwable.class)))) {
             addError("Catch statement parameter type is not a subclass of Throwable.", cs);
         }
         super.visitCatchStatement(cs);
     }
-    
+
     public void visitMethodCallExpression(MethodCallExpression mce) {
         super.visitMethodCallExpression(mce);
         Expression aexp = mce.getArguments();
         if (aexp instanceof TupleExpression) {
             TupleExpression arguments = (TupleExpression) aexp;
-            for (Iterator it=arguments.getExpressions().iterator();it. hasNext();) {
-                checkForInvalidDeclaration((Expression) it.next());
+            for (Expression e : arguments.getExpressions()) {
+                checkForInvalidDeclaration(e);
             }
         } else {
             checkForInvalidDeclaration(aexp);
         }
     }
-    
+
     private void checkForInvalidDeclaration(Expression exp) {
         if (!(exp instanceof DeclarationExpression)) return;
-        addError("invalid use of declartion inside method call.",exp);
+        addError("Invalid use of declaration inside method call.", exp);
     }
-    
+
     public void visitConstantExpression(ConstantExpression expression) {
         super.visitConstantExpression(expression);
         checkStringExceedingMaximumLength(expression);
     }
-    
+
     public void visitGStringExpression(GStringExpression expression) {
         super.visitGStringExpression(expression);
-        for (Iterator it = expression.getStrings().iterator(); it.hasNext();) {
-            checkStringExceedingMaximumLength((ConstantExpression) it.next());
+        for (ConstantExpression ce : expression.getStrings()) {
+            checkStringExceedingMaximumLength(ce);
         }
     }
-    
-    private void checkStringExceedingMaximumLength(ConstantExpression expression){
+
+    private void checkStringExceedingMaximumLength(ConstantExpression expression) {
         Object value = expression.getValue();
         if (value instanceof String) {
             String s = (String) value;
-            if (s.length()>65535) {
-                addError("String too long. The given string is "+s.length()+" Unicode code units long, but only a maximum of 65535 is allowed.",expression);
+            if (s.length() > 65535) {
+                addError("String too long. The given string is " + s.length() + " Unicode code units long, but only a maximum of 65535 is allowed.", expression);
             }
         }
     }
-    
+
 }
