@@ -26,6 +26,8 @@ import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.control.SourceUnit;
 
 import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
 
 /**
  * goes through an AST and initializes the scopes
@@ -34,8 +36,6 @@ import java.util.LinkedList;
  */
 public class VariableScopeVisitor extends ClassCodeVisitorSupport {
 
-    private static final Expression CALL = new ConstantExpression("call");
-
     private VariableScope currentScope = null;
     private VariableScope headScope = new VariableScope();
     private ClassNode currentClass = null;
@@ -402,13 +402,13 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     // ------------------------------
 
     public void visitClass(ClassNode node) {
-    	// AIC are already done, doing them here again will lead
-    	// to wrong scopes
-    	if (node instanceof InnerClassNode) {
-    		InnerClassNode in = (InnerClassNode) node;
-    		if (in.isAnonymous()) return;
-    	}
-    	
+        // AIC are already done, doing them here again will lead
+        // to wrong scopes
+        if (node instanceof InnerClassNode) {
+            InnerClassNode in = (InnerClassNode) node;
+            if (in.isAnonymous()) return;
+        }
+        
         pushState();
 
         currentClass = node;
@@ -454,7 +454,9 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
                 VariableExpression object = new VariableExpression(v);
                 object.setSourcePosition(methodNameConstant);
                 call.setObjectExpression(object);
-                call.setMethod(CALL);
+                ConstantExpression method = new ConstantExpression("call");
+                method.setSourcePosition(methodNameConstant); // important for GROOVY-4344
+                call.setMethod(method);
             }
 
         }
@@ -465,7 +467,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         isSpecialConstructorCall = call.isSpecialCall();
         super.visitConstructorCallExpression(call);
         isSpecialConstructorCall = false;
-        if (!call.isUsingAnnonymousInnerClass()) return;
+        if (!call.isUsingAnonymousInnerClass()) return;
 
         pushState();
         InnerClassNode innerClass = (InnerClassNode) call.getType();
@@ -505,4 +507,17 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         popState();
     }
 
+    public void visitAnnotations(AnnotatedNode node) {
+        List<AnnotationNode> annotations = node.getAnnotations();
+        if (annotations.isEmpty()) return;
+        ClassNode annType;
+        for (AnnotationNode an : annotations) {
+        	// skip built-in properties
+        	if (an.isBuiltIn()) continue;
+            for (Map.Entry<String, Expression> member : an.getMembers().entrySet()) {
+                Expression annMemberValue = member.getValue();
+                annMemberValue.visit(this);
+            }
+        }
+    }
 }
