@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 package org.codehaus.groovy.tools;
 
+import groovy.lang.GroovyResourceLoader;
 import org.apache.commons.cli.*;
 import org.codehaus.groovy.control.CompilationUnit;
 import org.codehaus.groovy.control.CompilerConfiguration;
@@ -28,12 +29,14 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
+import java.net.MalformedURLException;
+import java.net.URL;
 import java.util.*;
 
 /**
  * Command-line compiler (aka. <tt>groovyc</tt>).
  *
- * @version $Id: FileSystemCompiler.java 18415 2009-11-26 15:40:42Z roshandawrani $
+ * @version $Id: FileSystemCompiler.java 21452 2011-02-03 12:02:48Z ait $
  */
 public class FileSystemCompiler {
     private final CompilationUnit unit;
@@ -46,10 +49,17 @@ public class FileSystemCompiler {
         if (cu != null) {
             unit = cu;
         } else if (configuration.getJointCompilationOptions() != null) {
-            this.unit = new JavaAwareCompilationUnit(configuration);
+            unit = new JavaAwareCompilationUnit(configuration);
         } else {
-            this.unit = new CompilationUnit(configuration);
+            unit = new CompilationUnit(configuration);
         }
+
+        // in command line we don't need to do script lookups
+        unit.getClassLoader().setResourceLoader(new GroovyResourceLoader() {
+            public URL loadGroovySource(String filename) throws MalformedURLException {
+                return null;
+            }
+        });
     }
 
     public void compile(String[] paths) throws Exception {
@@ -70,7 +80,7 @@ public class FileSystemCompiler {
     public static void displayVersion() {
         String version = GroovySystem.getVersion();
         System.err.println("Groovy compiler version " + version);
-        System.err.println("Copyright 2003-2009 The Codehaus. http://groovy.codehaus.org/");
+        System.err.println("Copyright 2003-2010 The Codehaus. http://groovy.codehaus.org/");
         System.err.println("");
     }
 
@@ -244,27 +254,28 @@ public class FileSystemCompiler {
 
         Options options = new Options();
 
-        options.addOption(OptionBuilder.withLongOpt("classpath").hasArg().withArgName("path").withDescription("Specify where to find the class files.").create());
-        options.addOption(OptionBuilder.withLongOpt("sourcepath").hasArg().withArgName("path").withDescription("Specify where to find the source files.").create());
-        options.addOption(OptionBuilder.withLongOpt("temp").hasArg().withArgName("temp").withDescription("").create());
-        options.addOption(OptionBuilder.withLongOpt("encoding").hasArg().withArgName("encoding").withDescription("Specify the encoding of the user class files.").create());
-        options.addOption(OptionBuilder.hasArg().withDescription("Specify where to place generated class files.").create('d'));
+        options.addOption(OptionBuilder.hasArg().withArgName("path").withDescription("Specify where to find the class files - must be first argument").create("classpath"));
+        options.addOption(OptionBuilder.withLongOpt("classpath").hasArg().withArgName("path").withDescription("Aliases for '-classpath'").create("cp"));
+        options.addOption(OptionBuilder.withLongOpt("sourcepath").hasArg().withArgName("path").withDescription("Specify where to find the source files").create());
+        options.addOption(OptionBuilder.withLongOpt("temp").hasArg().withArgName("temp").withDescription("Specify temporary directory").create());
+        options.addOption(OptionBuilder.withLongOpt("encoding").hasArg().withArgName("encoding").withDescription("Specify the encoding of the user class files").create());
+        options.addOption(OptionBuilder.hasArg().withDescription("Specify where to place generated class files").create('d'));
 //            options.addOption(OptionBuilder.withLongOpt("strict").withDescription("Turn on strict type safety.").create('s'));
-        options.addOption(OptionBuilder.withLongOpt("help").withDescription("Print a synopsis of standard options.").create('h'));
-        options.addOption(OptionBuilder.withLongOpt("version").withDescription("Print the version.").create('v'));
-        options.addOption(OptionBuilder.withLongOpt("exception").withDescription("Print stack trace on error.").create('e'));
-        options.addOption(OptionBuilder.withLongOpt("jointCompilation").withDescription("Attach javac compiler to compile .java files.").create('j'));
+        options.addOption(OptionBuilder.withLongOpt("help").withDescription("Print a synopsis of standard options").create('h'));
+        options.addOption(OptionBuilder.withLongOpt("version").withDescription("Print the version").create('v'));
+        options.addOption(OptionBuilder.withLongOpt("exception").withDescription("Print stack trace on error").create('e'));
+        options.addOption(OptionBuilder.withLongOpt("jointCompilation").withDescription("Attach javac compiler to compile .java files").create('j'));
 
         options.addOption(
                 OptionBuilder.withArgName("property=value")
                         .withValueSeparator()
                         .hasArgs(2)
-                        .withDescription("")
+                        .withDescription("name-value pairs to pass to javac")
                         .create("J"));
         options.addOption(
                 OptionBuilder.withArgName("flag")
                         .hasArg()
-                        .withDescription("")
+                        .withDescription("passed to javac for joint compilation")
                         .create("F"));
         return options;
     }
