@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2010 the original author or authors.
+ * Copyright 2003-2011 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,9 +17,11 @@ package org.codehaus.groovy.tools.javac;
 
 import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.ast.expr.ArgumentListExpression;
+import org.codehaus.groovy.ast.expr.ClassExpression;
 import org.codehaus.groovy.ast.expr.ConstantExpression;
 import org.codehaus.groovy.ast.expr.ConstructorCallExpression;
 import org.codehaus.groovy.ast.expr.Expression;
+import org.codehaus.groovy.ast.expr.ListExpression;
 import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.expr.PropertyExpression;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
@@ -37,6 +39,7 @@ import java.io.IOException;
 import java.io.PrintWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -48,6 +51,7 @@ public class JavaStubGenerator
     private File outputPath;
     private List<String> toCompile = new ArrayList<String>();
     private ArrayList<MethodNode> propertyMethods = new ArrayList<MethodNode>();
+    private Map<String, MethodNode> propertyMethodsWithSigs = new HashMap<String, MethodNode>();
     private ArrayList<ConstructorNode> constructors = new ArrayList<ConstructorNode>();
     private ModuleNode currentModule;
 
@@ -125,14 +129,21 @@ public class JavaStubGenerator
                 protected void addTimeStamp(ClassNode node) {}
                 protected void addInitialization(ClassNode node) {}
                 protected void addPropertyMethod(MethodNode method) {
-                    propertyMethods.add(method);
+                	doAddMethod(method);
                 }
                 protected void addReturnIfNeeded(MethodNode node) {}
                 protected void addMethod(ClassNode node, boolean shouldBeSynthetic, String name, int modifiers, ClassNode returnType, Parameter[] parameters, ClassNode[] exceptions, Statement code) {
-                    propertyMethods.add(new MethodNode(name, modifiers, returnType, parameters, exceptions, code));
+                	doAddMethod(new MethodNode(name, modifiers, returnType, parameters, exceptions, code));
                 }
                 protected void addConstructor(Parameter[] newParams, ConstructorNode ctor, Statement code, ClassNode node) {
-                    constructors.add(new ConstructorNode(ctor.getModifiers(), newParams, ctor.getExceptions(), code));
+                	if(code instanceof ExpressionStatement) {//GROOVY-4508
+                		Statement temp = code;
+                		code = new BlockStatement();
+                		((BlockStatement)code).addStatement(temp);
+                	}
+                	ConstructorNode ctrNode = new ConstructorNode(ctor.getModifiers(), newParams, ctor.getExceptions(), code);
+                	ctrNode.setDeclaringClass(node);
+                	constructors.add(ctrNode);
                 }
                 protected void addDefaultParameters(DefaultArgsAction action, MethodNode method) {
                     final Parameter[] parameters = method.getParameters();
@@ -147,17 +158,29 @@ public class JavaStubGenerator
                             parameters[i].setInitialExpression(saved[i]);
                     }
                 }
+                private void doAddMethod(MethodNode method) {
+                	String sig = method.getTypeDescriptor();
+                	
+                	if(propertyMethodsWithSigs.containsKey(sig)) return;
+                	
+                	propertyMethods.add(method);
+                	propertyMethodsWithSigs.put(sig, method);
+                }
             };
             verifier.visitClass(classNode);
             currentModule = classNode.getModule();
 
             boolean isInterface = classNode.isInterface();
             boolean isEnum = (classNode.getModifiers() & Opcodes.ACC_ENUM) !=0;
+            boolean isAnnotationDefinition = classNode.isAnnotationDefinition();
             printAnnotations(out, classNode);
             printModifiers(out, classNode.getModifiers()
                     & ~(isInterface ? Opcodes.ACC_ABSTRACT : 0));
 
             if (isInterface) {
+            	if(isAnnotationDefinition) {
+            		out.print ("@");
+            	}
                 out.print ("interface ");
             } else if (isEnum) {
                 out.print ("enum ");
@@ -179,7 +202,7 @@ public class JavaStubGenerator
             }
 
             ClassNode[] interfaces = classNode.getInterfaces();
-            if (interfaces != null && interfaces.length > 0) {
+            if (interfaces != null && interfaces.length > 0 && !isAnnotationDefinition) {
                 if (isInterface) {
                     out.println("  extends");
                 } else {
@@ -201,6 +224,7 @@ public class JavaStubGenerator
             for (Iterator<InnerClassNode> inner = classNode.getInnerClasses(); inner.hasNext();) {
                 // GROOVY-4004: Clear the methods from the outer class so that they don't get duplicated in inner ones
                 propertyMethods.clear();
+                propertyMethodsWithSigs.clear();
                 constructors.clear();
                 genClassInner(inner.next(), out);
             }
@@ -209,6 +233,7 @@ public class JavaStubGenerator
         }
         finally {
             propertyMethods.clear();
+            propertyMethodsWithSigs.clear();
             constructors.clear();
             currentModule = null;
         }
@@ -440,6 +465,7 @@ public class JavaStubGenerator
     private void genMethod(ClassNode clazz, MethodNode methodNode, PrintWriter out) {
         if (methodNode.getName().equals("<clinit>")) return;
         if (methodNode.isPrivate() || !Utilities.isJavaIdentifier(methodNode.getName())) return;
+        if (methodNode.isSynthetic() && methodNode.getName().equals("$getStaticMetaClass")) return;
 
         printAnnotations(out, methodNode);
         if (!clazz.isInterface()) printModifiers(out, methodNode.getModifiers());
@@ -485,7 +511,7 @@ public class JavaStubGenerator
     }
 
     private void printDefaultValue(PrintWriter out, ClassNode type) {
-        if (type.redirect()!=ClassHelper.OBJECT_TYPE) {
+        if (type.redirect() != ClassHelper.OBJECT_TYPE && type.redirect() != ClassHelper.boolean_TYPE) {
             out.print("(");
             printType(type,out);
             out.print(")");
@@ -610,30 +636,43 @@ public class JavaStubGenerator
             boolean first = true;
             Map<String, Expression> members = annotation.getMembers();
             for (String key : members.keySet()) {
-                String val = "null";
-                Object memberValue = members.get(key);
-                if (memberValue instanceof ConstantExpression) {
-                    ConstantExpression ce = (ConstantExpression) memberValue;
-                    Object constValue = ce.getValue();
-                    if (constValue instanceof Number || constValue instanceof Boolean)
-                        val = constValue.toString();
-                    else
-                        val = "\"" + constValue + "\"";
-                } else if (memberValue instanceof PropertyExpression || memberValue instanceof VariableExpression) {
-                    // assume must be static class field or enum value or class that Java can resolve
-                    val = ((Expression) memberValue).getText();
-                }
-                if (first) {
-                    first = false;
-                } else {
-                    out.print(", ");
-                }
-                out.print(key + "=" + val);
+                if (first) first = false;
+                else out.print(", ");
+                out.print(key + "=" + getAnnotationValue(members.get(key)));
             }
             out.print(") ");
         }
     }
 
+    private String getAnnotationValue(Object memberValue) {
+        String val = "null";
+        if (memberValue instanceof ListExpression) {
+            StringBuilder sb = new StringBuilder("{");
+            boolean first = true;
+            ListExpression le = (ListExpression) memberValue;
+            for (Expression e : le.getExpressions()) {
+                if (first) first = false;
+                else sb.append(",");
+                sb.append(getAnnotationValue(e));
+            }
+            sb.append("}");
+            val = sb.toString();
+        } else if (memberValue instanceof ConstantExpression) {
+            ConstantExpression ce = (ConstantExpression) memberValue;
+            Object constValue = ce.getValue();
+            if (constValue instanceof Number || constValue instanceof Boolean)
+                val = constValue.toString();
+            else
+				val = "\"" + escapeStringAnnotationValue(constValue.toString()) + "\"";
+        } else if (memberValue instanceof PropertyExpression || memberValue instanceof VariableExpression) {
+            // assume must be static class field or enum value or class that Java can resolve
+            val = ((Expression) memberValue).getText();
+        } else if (memberValue instanceof ClassExpression) {
+        	val = ((Expression) memberValue).getText() + ".class";
+        }
+        return val;
+    }
+
     private void printModifiers(PrintWriter out, int modifiers) {
         if ((modifiers & Opcodes.ACC_PUBLIC) != 0)
             out.print("public ");
@@ -663,10 +702,10 @@ public class JavaStubGenerator
         }
 
         for (ImportNode imp : moduleNode.getImports()) {
-            imports.add(imp.getType().getName());
+            if (imp.getAlias() == null)
+                imports.add(imp.getType().getName());
         }
 
-        // TODO: find out why everything isn't fully resolved by now then delete next line
         imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
 
         for (String imp : imports) {
@@ -687,4 +726,8 @@ public class JavaStubGenerator
             new File(outputPath, path + ".java").delete();
         }
     }
+
+	private static String escapeStringAnnotationValue(String value) {
+		return value.replace("\n", "\\n").replace("\r", "\\r").replace("\"", "\\\"");
+	}
 }
