@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2009 the original author or authors.
+ * Copyright 2008-2010 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,20 +27,32 @@ import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.syntax.SyntaxException;
 import org.objectweb.asm.Opcodes;
 
-import java.util.Set;
-import java.util.LinkedList;
-import java.util.HashSet;
+import groovy.lang.Reference;
+
 import java.util.Arrays;
+import java.util.HashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Set;
 
 /**
- * Handles generation of code for the @Category annotation
- * - all non-static methods converted to static ones with additional parameter 'self'
+ * Handles generation of code for the @Category annotation.
+ * <p>
+ * Transformation logic is as follows:
+ * <ul>
+ * <li>all non-static methods converted to static ones with an additional 'self' parameter
+ * <li>references to 'this' changed to the additional 'self' parameter
+ * </ul>
  *
  * @author Alex Tkachman
  */
 @GroovyASTTransformation(phase = CompilePhase.CANONICALIZATION)
 public class CategoryASTTransformation implements ASTTransformation, Opcodes {
-    private static final VariableExpression THIS_EXPRESSION = new VariableExpression("$this");
+    private static final VariableExpression THIS_EXPRESSION;
+    static {
+        THIS_EXPRESSION = new VariableExpression("$this");
+        THIS_EXPRESSION.setClosureSharedVariable(true);
+    }
 
     /**
      * Property invocations done on 'this' reference are transformed so that the invocations at runtime are
@@ -62,24 +74,29 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
             names.add(field.getName());
         }
         varStack.add(names);
-
+        final Reference parameter = new Reference();
         final ClassCodeExpressionTransformer expressionTransformer = new ClassCodeExpressionTransformer() {
             protected SourceUnit getSourceUnit() {
                 return source;
             }
 
-            public void visitMethod(MethodNode node) {
+            private void addVariablesToStack(Parameter[] params) {
                 Set<String> names = new HashSet<String>();
                 names.addAll(varStack.getLast());
-                final Parameter[] params = node.getParameters();
                 for (Parameter param : params) {
                     names.add(param.getName());
                 }
                 varStack.add(names);
+            }
+            
+            @Override
+            public void visitMethod(MethodNode node) {
+                addVariablesToStack(node.getParameters());
                 super.visitMethod(node);
                 varStack.removeLast();
             }
 
+            @Override
             public void visitBlockStatement(BlockStatement block) {
                 Set<String> names = new HashSet<String>();
                 names.addAll(varStack.getLast());
@@ -88,11 +105,30 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
                 varStack.remove(names);
             }
 
+            @Override
+            public void visitClosureExpression(ClosureExpression ce) {
+                addVariablesToStack(ce.getParameters());
+                super.visitClosureExpression(ce);
+                varStack.removeLast();
+            }
+
+            @Override
             public void visitDeclarationExpression(DeclarationExpression expression) {
-                varStack.getLast().add(expression.getVariableExpression().getName());
+                if (expression.isMultipleAssignmentDeclaration()) {
+                    TupleExpression te = (TupleExpression) expression.getLeftExpression();
+                    List<Expression> list = te.getExpressions();
+                    for (Expression arg : list) {
+                        VariableExpression ve = (VariableExpression) arg;
+                        varStack.getLast().add(ve.getName());
+                    }
+                } else {
+                    VariableExpression ve = expression.getVariableExpression();
+                    varStack.getLast().add(ve.getName());
+                }
                 super.visitDeclarationExpression(expression);
             }
 
+            @Override
             public void visitForLoop(ForStatement forLoop) {
                 Expression exp = forLoop.getCollectionExpression();
                 exp.visit(this);
@@ -103,6 +139,7 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
                 super.visitForLoop(forLoop);
             }
 
+            @Override
             public void visitExpressionStatement(ExpressionStatement es) {
                 // GROOVY-3543: visit the declaration expressions so that declaration variables get added on the varStack
                 Expression exp = es.getExpression();
@@ -112,6 +149,7 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
                 super.visitExpressionStatement(es);
             }
 
+            @Override
             public Expression transform(Expression exp) {
                 if (exp instanceof VariableExpression) {
                     VariableExpression ve = (VariableExpression) exp;
@@ -131,6 +169,20 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
                             return pe;
                         }
                     }
+                } else if (exp instanceof ClosureExpression) {
+                    ClosureExpression ce = (ClosureExpression) exp;
+                    ce.getVariableScope().putReferencedLocalVariable((Parameter) parameter.get());
+                    Parameter[] params = ce.getParameters();
+                    if (params==null){
+                        params = new Parameter[0];
+                    } else if (params.length==0) {
+                        params = new Parameter[] {
+                                new Parameter(ClassHelper.OBJECT_TYPE, "it")
+                        };
+                    }
+                    addVariablesToStack(params);
+                    ce.getCode().visit(this);
+                    varStack.removeLast();
                 }
                 return super.transform(exp);
             }
@@ -141,7 +193,10 @@ public class CategoryASTTransformation implements ASTTransformation, Opcodes {
                 method.setModifiers(method.getModifiers() | Opcodes.ACC_STATIC);
                 final Parameter[] origParams = method.getParameters();
                 final Parameter[] newParams = new Parameter[origParams.length + 1];
-                newParams[0] = new Parameter(targetClass, "$this");
+                Parameter p = new Parameter(targetClass, "$this");
+                p.setClosureSharedVariable(true);
+                newParams[0] = p; 
+                parameter.set(p);
                 System.arraycopy(origParams, 0, newParams, 1, origParams.length);
                 method.setParameters(newParams);
 
