@@ -15,16 +15,17 @@
  */
 package org.codehaus.groovy.transform;
 
-import groovy.lang.GroovyObject;
 import groovy.lang.Immutable;
 import groovy.lang.MetaClass;
 import groovy.lang.MissingPropertyException;
+import groovy.lang.ReadOnlyPropertyException;
 import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.*;
 import org.codehaus.groovy.control.CompilePhase;
 import org.codehaus.groovy.control.SourceUnit;
 import org.codehaus.groovy.runtime.DefaultGroovyMethods;
+import org.codehaus.groovy.runtime.InvokerHelper;
 import org.codehaus.groovy.syntax.Token;
 import org.codehaus.groovy.syntax.Types;
 import org.codehaus.groovy.util.HashCodeHelper;
@@ -34,7 +35,6 @@ import java.util.*;
 
 /**
  * Handles generation of code for the @Immutable annotation.
- * This is experimental, use at your own risk.
  *
  * @author Paul King
  */
@@ -48,21 +48,21 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
       Also, Color is not final so while not normally used with child
       classes, it isn't strictly immutable. Use at your own risk.
      */
-    private static Class[] immutableList = {
-            Boolean.class,
-            Byte.class,
-            Character.class,
-            Double.class,
-            Float.class,
-            Integer.class,
-            Long.class,
-            Short.class,
-            String.class,
-            java.math.BigInteger.class,
-            java.math.BigDecimal.class,
-            java.awt.Color.class,
-            java.net.URI.class,
-    };
+    private static List<String> immutableList = Arrays.asList(
+            "java.lang.Boolean",
+            "java.lang.Byte",
+            "java.lang.Character",
+            "java.lang.Double",
+            "java.lang.Float",
+            "java.lang.Integer",
+            "java.lang.Long",
+            "java.lang.Short",
+            "java.lang.String",
+            "java.math.BigInteger",
+            "java.math.BigDecimal",
+            "java.awt.Color",
+            "java.net.URI"
+    );
     private static final Class MY_CLASS = Immutable.class;
     private static final ClassNode MY_TYPE = new ClassNode(MY_CLASS);
     private static final String MY_TYPE_NAME = "@" + MY_TYPE.getNameWithoutPackage();
@@ -74,7 +74,9 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     private static final ClassNode COLLECTION_TYPE = new ClassNode(Collection.class);
     private static final ClassNode HASHUTIL_TYPE = new ClassNode(HashCodeHelper.class);
     private static final ClassNode STRINGBUFFER_TYPE = new ClassNode(StringBuffer.class);
+    private static final ClassNode READONLYEXCEPTION_TYPE = new ClassNode(ReadOnlyPropertyException.class);
     private static final ClassNode DGM_TYPE = new ClassNode(DefaultGroovyMethods.class);
+    private static final ClassNode INVOKER_TYPE = new ClassNode(InvokerHelper.class);
     private static final ClassNode SELF_TYPE = new ClassNode(ImmutableASTTransformation.class);
     private static final Token COMPARE_EQUAL = Token.newSymbol(Types.COMPARE_EQUAL, -1, -1);
     private static final Token COMPARE_NOT_EQUAL = Token.newSymbol(Types.COMPARE_NOT_EQUAL, -1, -1);
@@ -145,7 +147,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
 
         final FieldNode hashField = cNode.addField("$hash$code", ACC_PRIVATE | ACC_SYNTHETIC, ClassHelper.int_TYPE, null);
         final BlockStatement body = new BlockStatement();
-        final Expression hash = new FieldExpression(hashField);
+        final Expression hash = new VariableExpression(hashField);
         final List<PropertyNode> list = getInstanceProperties(cNode);
 
         body.addStatement(new IfStatement(
@@ -182,12 +184,12 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
                 body.addStatement(append(result, new ConstantExpression(", ")));
             }
             body.addStatement(new IfStatement(
-                    new BooleanExpression(new FieldExpression(cNode.getField("$map$constructor"))),
+                    new BooleanExpression(new VariableExpression(cNode.getField("$map$constructor"))),
                     toStringPropertyName(result, pNode.getName()),
                     new EmptyStatement()
             ));
-            final FieldExpression fieldExpr = new FieldExpression(pNode.getField());
-            body.addStatement(append(result, new MethodCallExpression(fieldExpr, "toString", MethodCallExpression.NO_ARGUMENTS)));
+            final Expression fieldExpr = new VariableExpression(pNode.getField());
+            body.addStatement(append(result, new StaticMethodCallExpression(INVOKER_TYPE, "toString", fieldExpr)));
         }
         body.addStatement(append(result, new ConstantExpression(")")));
         body.addStatement(new ReturnStatement(new MethodCallExpression(result, "toString", MethodCallExpression.NO_ARGUMENTS)));
@@ -216,7 +218,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         // fields
         for (PropertyNode pNode : list) {
             // _result = HashCodeHelper.updateHash(_result, field)
-            final Expression fieldExpr = new FieldExpression(pNode.getField());
+            final Expression fieldExpr = new VariableExpression(pNode.getField());
             final Expression args = new TupleExpression(result, fieldExpr);
             final Expression current = new StaticMethodCallExpression(HASHUTIL_TYPE, "updateHash", args);
             body.addStatement(assignStatement(result, current));
@@ -300,7 +302,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     private void createConstructor(ClassNode cNode) {
         // pretty toString will remember how the user declared the params and print accordingly
         final FieldNode constructorField = cNode.addField("$map$constructor", ACC_PRIVATE | ACC_SYNTHETIC, ClassHelper.boolean_TYPE, null);
-        final FieldExpression constructorStyle = new FieldExpression(constructorField);
+        final Expression constructorStyle = new VariableExpression(constructorField);
         if (cNode.getDeclaredConstructors().size() != 0) {
             // TODO: allow constructors which call provided constructor?
             throw new RuntimeException("Explicit constructors not allowed for " + MY_TYPE_NAME + " class: " + cNode.getNameWithoutPackage());
@@ -326,13 +328,13 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         return result;
     }
 
-    private void createConstructorMapSpecial(ClassNode cNode, FieldExpression constructorStyle, List<PropertyNode> list) {
+    private void createConstructorMapSpecial(ClassNode cNode, Expression constructorStyle, List<PropertyNode> list) {
         final BlockStatement body = new BlockStatement();
         body.addStatement(createConstructorStatementMapSpecial(list.get(0).getField()));
         createConstructorMapCommon(cNode, constructorStyle, body);
     }
 
-    private void createConstructorMap(ClassNode cNode, FieldExpression constructorStyle, List<PropertyNode> list) {
+    private void createConstructorMap(ClassNode cNode, Expression constructorStyle, List<PropertyNode> list) {
         final BlockStatement body = new BlockStatement();
         for (PropertyNode pNode : list) {
             body.addStatement(createConstructorStatement(cNode, pNode));
@@ -343,12 +345,15 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         createConstructorMapCommon(cNode, constructorStyle, body);
     }
 
-    private void createConstructorMapCommon(ClassNode cNode, FieldExpression constructorStyle, BlockStatement body) {
+    private void createConstructorMapCommon(ClassNode cNode, Expression constructorStyle, BlockStatement body) {
         final List<FieldNode> fList = cNode.getFields();
         for (FieldNode fNode : fList) {
-            if (!fNode.isPublic() && !fNode.getName().contains("$") && (cNode.getProperty(fNode.getName()) == null)) {
-                body.addStatement(createConstructorStatementDefault(fNode));
-            }
+            if (fNode.isPublic()) continue; // public fields will be rejected elsewhere
+            if (cNode.getProperty(fNode.getName()) != null) continue; // a property
+            if (fNode.isFinal() && fNode.isStatic()) continue;
+            if (fNode.getName().contains("$")) continue; // internal field
+            if (fNode.isFinal() && fNode.getInitialExpression() != null) body.addStatement(checkFinalArgNotOverridden(cNode, fNode));
+            body.addStatement(createConstructorStatementDefault(fNode));
         }
         body.addStatement(assignStatement(constructorStyle, ConstantExpression.TRUE));
         final Parameter[] params = new Parameter[]{new Parameter(HASHMAP_TYPE, "args")};
@@ -358,7 +363,18 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
                 body)));
     }
 
-    private void createConstructorOrdered(ClassNode cNode, FieldExpression constructorStyle, List<PropertyNode> list) {
+    private Statement checkFinalArgNotOverridden(ClassNode cNode, FieldNode fNode) {
+        final String name = fNode.getName();
+        Expression value = findArg(name);
+        return new IfStatement(
+                equalsNullExpr(value),
+                new EmptyStatement(),
+                new ThrowStatement(new ConstructorCallExpression(READONLYEXCEPTION_TYPE,
+                        new ArgumentListExpression(new ConstantExpression(name),
+                                new ConstantExpression(cNode.getName())))));
+    }
+
+    private void createConstructorOrdered(ClassNode cNode, Expression constructorStyle, List<PropertyNode> list) {
         final MapExpression argMap = new MapExpression();
         final Parameter[] orderedParams = new Parameter[list.size()];
         int index = 0;
@@ -378,11 +394,11 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         FieldNode fNode = pNode.getField();
         final ClassNode fieldType = fNode.getType();
         final Statement statement;
-        if (fieldType.isArray() || implementsInterface(fieldType, CLONEABLE_TYPE)) {
+        if (fieldType.isArray() || fieldType.implementsInterface(CLONEABLE_TYPE)) {
             statement = createConstructorStatementArrayOrCloneable(fNode);
         } else if (fieldType.isDerivedFrom(DATE_TYPE)) {
             statement = createConstructorStatementDate(fNode);
-        } else if (fieldType.isDerivedFrom(COLLECTION_TYPE) || fieldType.isDerivedFrom(MAP_TYPE)) {
+        } else if (isOrImplements(fieldType, COLLECTION_TYPE) || isOrImplements(fieldType, MAP_TYPE)) {
             statement = createConstructorStatementCollection(fNode);
         } else if (isKnownImmutable(fieldType)) {
             statement = createConstructorStatementDefault(fNode);
@@ -394,12 +410,12 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         return statement;
     }
 
-    private boolean implementsInterface(ClassNode fieldType, ClassNode interfaceType) {
-        return Arrays.asList(fieldType.getInterfaces()).contains(interfaceType);
+    private boolean isOrImplements(ClassNode fieldType, ClassNode interfaceType) {
+        return fieldType.equals(interfaceType) || fieldType.implementsInterface(interfaceType);
     }
 
     private Statement createConstructorStatementGuarded(ClassNode cNode, FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
         Expression unknown = findArg(fNode.getName());
@@ -418,7 +434,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createConstructorStatementCollection(FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
         Expression collection = findArg(fNode.getName());
@@ -432,7 +448,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createConstructorStatementMapSpecial(FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
         Expression namedArgs = findArg(fNode.getName());
@@ -459,21 +475,21 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
 
     private boolean isKnownImmutable(ClassNode fieldType) {
         if (!fieldType.isResolved()) return false;
-        Class typeClass = fieldType.getTypeClass();
-        return typeClass.isEnum() ||
-                typeClass.isPrimitive() ||
-                inImmutableList(typeClass);
+        return fieldType.isEnum() ||
+                ClassHelper.isPrimitiveType(fieldType) ||
+                inImmutableList(fieldType.getName());
     }
 
-    private static boolean inImmutableList(Class typeClass) {
-        return Arrays.asList(immutableList).contains(typeClass);
+    private static boolean inImmutableList(String typeName) {
+        return immutableList.contains(typeName);
     }
 
     private Statement createConstructorStatementDefault(FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final String name = fNode.getName();
+        final Expression fieldExpr = new PropertyExpression(VariableExpression.THIS_EXPRESSION, name);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
-        Expression value = findArg(fNode.getName());
+        final Expression value = findArg(fNode.getName());
         return new IfStatement(
                 equalsNullExpr(value),
                 new IfStatement(
@@ -484,7 +500,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createConstructorStatementArrayOrCloneable(FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
         final Expression array = findArg(fNode.getName());
@@ -498,7 +514,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createConstructorStatementDate(FieldNode fNode) {
-        final FieldExpression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         Expression initExpr = fNode.getInitialValueExpression();
         if (initExpr == null) initExpr = ConstantExpression.NULL;
         final Expression date = findArg(fNode.getName());
@@ -541,7 +557,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private BooleanExpression notEqualsExpr(PropertyNode pNode, Expression other) {
-        final Expression fieldExpr = new FieldExpression(pNode.getField());
+        final Expression fieldExpr = new VariableExpression(pNode.getField());
         final Expression otherExpr = new PropertyExpression(other, pNode.getField().getName());
         return new BooleanExpression(new BinaryExpression(fieldExpr, COMPARE_NOT_EQUAL, otherExpr));
     }
@@ -575,7 +591,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
         BlockStatement body = new BlockStatement();
         final ClassNode fieldType = fNode.getType();
         final Statement statement;
-        if (fieldType.isArray() || implementsInterface(fieldType, CLONEABLE_TYPE)) {
+        if (fieldType.isArray() || fieldType.implementsInterface(CLONEABLE_TYPE)) {
             statement = createGetterBodyArrayOrCloneable(fNode);
         } else if (fieldType.isDerivedFrom(DATE_TYPE)) {
             statement = createGetterBodyDate(fNode);
@@ -587,7 +603,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createGetterBodyDefault(FieldNode fNode) {
-        final Expression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         return new ExpressionStatement(fieldExpr);
     }
 
@@ -607,7 +623,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createGetterBodyArrayOrCloneable(FieldNode fNode) {
-        final Expression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         final Expression expression = cloneArrayOrCloneableExpr(fieldExpr);
         return safeExpression(fieldExpr, expression);
     }
@@ -621,7 +637,7 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     private Statement createGetterBodyDate(FieldNode fNode) {
-        final Expression fieldExpr = new FieldExpression(fNode);
+        final Expression fieldExpr = new VariableExpression(fNode);
         final Expression expression = cloneDateExpr(fieldExpr);
         return safeExpression(fieldExpr, expression);
     }
@@ -634,15 +650,15 @@ public class ImmutableASTTransformation implements ASTTransformation, Opcodes {
     }
 
     public static Object checkImmutable(String className, String fieldName, Object field) {
-        if (field == null || field instanceof Enum || inImmutableList(field.getClass())) return field;
+        if (field == null || field instanceof Enum || inImmutableList(field.getClass().getName())) return field;
         if (field instanceof Collection) return DefaultGroovyMethods.asImmutable((Collection) field);
         if (field.getClass().getAnnotation(MY_CLASS) != null) return field;
         final String typeName = field.getClass().getName();
         throw new RuntimeException(createErrorMessage(className, fieldName, typeName, "constructing"));
     }
 
-    public static void checkPropNames(GroovyObject instance, Map<String, Object> args) {
-        final MetaClass metaClass = instance.getMetaClass();
+    public static void checkPropNames(Object instance, Map<String, Object> args) {
+        final MetaClass metaClass = InvokerHelper.getMetaClass(instance);
         for (String k : args.keySet()) {
             if (metaClass.hasProperty(instance, k) == null)
                 throw new MissingPropertyException(k, instance.getClass());
